@@ -30,9 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if !defined(_WIN32) && !defined(_WIN64)
 #include <unistd.h>
-#endif
 
 #if defined(SERIAL_SUPPORT)
 #if !defined(_WIN32) && !defined(_WIN64)
@@ -49,12 +47,6 @@ typedef long speed_t;
      "by this build of GPX"
 #endif
 
-#if defined(_WIN32) || defined(_WIN64)
-#include "getopt_.h"
-// strcasecmp() is not provided on Windows
-// _stricmp() is equivalent but may require <string.h>
-#define strcasecmp _stricmp
-#endif
 
 #include "gpx.h"
 #include "machine_config.h"
@@ -74,19 +66,20 @@ static char temp_config_name[24];
 static void exit_handler(void)
 {
     // close open files
-    if(file_in != stdin) {
+    if(file_in != stdin && file_in != NULL) {
         fclose(file_in);
-        if(file_out != stdout) {
-            if(ferror(file_out)) {
-                perror("Error writing to output file");
-            }
-            fclose(file_out);
-	    file_out = NULL;
-        }
-        if(file_out2) {
-            fclose(file_out2);
-	    file_out2 = NULL;
+        file_in = NULL;
+    }
+    if(file_out != stdout && file_out != NULL) {
+        if(ferror(file_out)) {
+            perror("Error writing to output file");
         }
+        fclose(file_out);
+        file_out = NULL;
+    }
+    if(file_out2 != NULL) {
+        fclose(file_out2);
+        file_out2 = NULL;
     }
 
     // 23 February 2015
@@ -145,11 +138,12 @@ static void usage(int err)
     fputs("GNU General Public License for more details." EOL, fp);
 
     fputs(EOL "Usage:" EOL, fp);
-    fputs("gpx [-CFdgilpqr" SERIAL_MSG1 "tvw] " SERIAL_MSG2 "[-b BAUDRATE] [-c CONFIG] [-e EEPROM] [-f DIAMETER] [-m MACHINE] [-N h|t|ht] [-n SCALE] [-x X] [-y Y] [-z Z] IN [OUT]" EOL, fp);
+    fputs("gpx [-CFIdgilpqr" SERIAL_MSG1 "tvw] " SERIAL_MSG2 "[-b BAUDRATE] [-c CONFIG] [-e EEPROM] [-f DIAMETER] [-m MACHINE] [-N h|t|ht] [-n SCALE] [-x X] [-y Y] [-z Z] IN [OUT]" EOL, fp);
     fputs(EOL "Options:" EOL, fp);
-    fputs("\t-C\tCreate temporary file with a copy of the machine configuration" EOL, fp);
+    fputs("\t-C\tcreate temporary file with a copy of the machine configuration" EOL, fp);
     fputs("\t-F\twrite X3G on-wire framing data to output file" EOL, fp);
-    fputs("\t-N\tDisable writing of the X3G header (start build notice)," EOL, fp);
+    fputs("\t-I\tignore default .ini files" EOL, fp);
+    fputs("\t-N\tdisable writing of the X3G header (start build notice)," EOL, fp);
     fputs("\t  \ttail (end build notice), or both" EOL, fp);
     fputs("\t-d\tsimulated ditto printing" EOL, fp);
     fputs("\t-g\tMakerbot/ReplicatorG GCODE flavor" EOL, fp);
@@ -355,6 +349,7 @@ int main(int argc, char * const argv[])
 {
     int c, i, rval = 1;
     int force_framing = 0;
+    int ignore_default_ini = 0;
     int log_to_file = 0;
     int standard_io = 0;
     int serial_io = 0;
@@ -363,9 +358,6 @@ int main(int argc, char * const argv[])
     char *eeprom = NULL;
     double filament_diameter = 0;
     char *buildname = PACKAGE_STRING;
-#ifdef _WIN32
-    char *otherdelim = NULL;
-#endif
     char *filename;
     speed_t baud_rate = B115200;
     int make_temp_config = 0;
@@ -384,16 +376,36 @@ int main(int argc, char * const argv[])
     gpx_initialize(&gpx, 1);
     gpx.log = stderr;
 
-    // peek at first option in case we want verbose for reading gpx.ini
-    if (argc > 1 && argv[1][0] && argv[1][1] == 'v')
-        gpx.flag.verboseMode = 1;
+    // we run through getopt twice, the first time is to figure out whether to load
+    // the ini file from the default locations and whether to be verbose about it
+    // we need to load the ini file before parsing the rest so that the command line
+    // overrides the default ini in the standard case
+    while ((c = getopt(argc, argv, "CFIN:b:c:de:gf:ilm:n:pqrstu:vwx:y:z:?")) != -1) {
+        switch (c) {
+            case 'I':
+                ignore_default_ini = 1;
+                break;
+            case 'l':
+                gpx.flag.verboseMode = 1;
+                log_to_file = 1;
+                break;
+            case 'v':
+                if(gpx.flag.verboseMode)
+                    gpx.flag.verboseSioMode = 1;
+                gpx.flag.verboseMode = 1;
+                break;
+        }
+    }
+    optind = 1;
 
-    // READ GPX.INI
-    i = gpx_find_ini(&gpx, argv[0]);
-    if (i > 0) {
-        fprintf(stderr, "(line %u) Configuration syntax error: unrecognised parameters" EOL, i);
-        usage(1);
-        goto done;
+    if(!ignore_default_ini) {
+        // READ GPX.INI
+        i = gpx_find_ini(&gpx, argv[0]);
+        if (i > 0) {
+            fprintf(stderr, "(line %u) Configuration syntax error: unrecognised parameters" EOL, i);
+            usage(1);
+            goto done;
+        }
     }
 
     // READ COMMAND LINE
@@ -403,7 +415,7 @@ int main(int argc, char * const argv[])
     // error message should they be attempted when the code
     // is compiled without serial I/O support.
 
-    while ((c = getopt(argc, argv, "CFN:b:c:de:gf:ilm:n:pqrstu:vwx:y:z:?")) != -1) {
+    while ((c = getopt(argc, argv, "CFIN:b:c:de:gf:ilm:n:pqrstu:vwx:y:z:?")) != -1) {
         switch (c) {
 	    case 'C':
 		 // Write config data to a temp file
@@ -413,6 +425,8 @@ int main(int argc, char * const argv[])
 	    case 'F':
 		 force_framing = ITEM_FRAMING_ENABLE;
 		 break;
+            case 'I':
+                 break; // handled in first getopt loop
 	    case 'N':
 		 if(optarg[0] == 'h' || optarg[1] == 'h')
 		      gpx_set_start(&gpx, 0);
@@ -465,7 +479,7 @@ int main(int argc, char * const argv[])
                 // fall through
             case 's':
 #if !defined(SERIAL_SUPPORT)
-		 fprintf(stderr, NO_SERIAL_SUPPORT_MSG EOL);
+		fprintf(stderr, NO_SERIAL_SUPPORT_MSG EOL);
 		usage(1);
 		goto done;
 #else
@@ -496,9 +510,7 @@ int main(int argc, char * const argv[])
                 standard_io = 1;
                 break;
             case 'l':
-                gpx.flag.verboseMode = 1;
-                log_to_file = 1;
-                break;
+                break; // handled in first getopt loop
             case 'm':
                 if(gpx_set_property(&gpx, "printer", "machine_type", optarg)) {
                     usage(1);
@@ -527,10 +539,7 @@ int main(int argc, char * const argv[])
                 }
                 break;
             case 'v':
-                if (gpx.flag.verboseMode)
-                    gpx.flag.verboseSioMode = 1;
-                gpx.flag.verboseMode = 1;
-                break;
+                break; // handled in first getopt loop
             case 'w':
                 gpx.flag.rewrite5D = 1;
                 break;
